unit mdldatadict;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpdatadict;

Type

  { TModelComponent }

  TModelComponent = Class(TComponent)
  Private
    FDims : Array[0..3] of Integer;
    function GetD(AIndex: integer): integer;
    procedure SetD(AIndex: integer; const AValue: integer);
  protected
    Procedure InvalidateRect(Erase : boolean);
  Public
    function HasParent: Boolean; override;
    function GetParentComponent: TComponent; override;
    Procedure SetBounds(Const ALeft,ATop,AWidth,AHeight : Integer);
  Published
    Property Top : integer Index 0 Read GetD Write SetD;
    Property Left : integer Index 1 Read GetD Write SetD;
    Property Width : integer Index 2 Read GetD Write SetD;
    Property Height : integer Index 3 Read GetD Write SetD;
  end;

  { TDDModelPage }

  TOnInvalidateRect = Procedure (Sender : TObject; Const Rect : TRect; Erase: boolean) of object;

  TDDModelPage = Class(TModelComponent)
  private
    FOnInvalidateRect: TOnInvalidateRect;
    FPageName: String;
  Protected
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    Procedure DoInvalidateRect(Const ARect : TRect; Erase: boolean); virtual;
  Public
    Property OnInvalidateRect : TOnInvalidateRect Read FOnInvalidateRect Write FOnInvalidateRect;
  Published
    Property PageName : String Read FPageName Write FPageName;
  end;

  { TModelFieldDef }
{$define oldfields}
{$ifdef oldfields}
  TModelFieldDef = TDDFieldDef;
{$else}
  TModelFieldDef = Class(TDDFieldDef)
  Protected
    function GetDisplayName: string; override;
  end;
{$endif}

  { TModelFieldDefs }

  TModelFieldDefs = Class(TDDFieldDefs)
  private
    function GetF(Aindex : Integer): TModelFieldDef;
    procedure SetF(Aindex : Integer; const AValue: TModelFieldDef);
  Public
    Property Fields[Aindex : Integer] : TModelFieldDef Read GetF Write SetF; default;
  end;


  { TModelText }
  TModelText = Class(TModelComponent)
  private
    FCaption: String;
    FText: TStrings;
    function GetCaption: String;
    procedure SetText(const AValue: TStrings);
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
  Published
    Property Caption : String Read GetCaption Write FCaption;
    Property Text : TStrings Read FText Write SetText;
  end;

  { TModelTable }
  TModelTable = Class(TModelComponent)
  private
    FFields: TModelFieldDefs;
    FTableName : String;
    function GetTableName: String;
    procedure SetFields(const AValue: TModelFieldDefs);
    procedure SetTableName(const AValue: String);
  Public
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
  Published
    Property TableName : String Read GetTableName Write SetTableName;
    Property Fields : TModelFieldDefs Read FFields Write SetFields;
  end;

implementation

{ TModelTable }

function TModelTable.GetTableName: String;
begin
  Result:=FTableName;
  If (Result='') then
    Result:=Name;
end;

procedure TModelTable.SetFields(const AValue: TModelFieldDefs);
begin
  if FFields=AValue then exit;
  FFields.Assign(AValue);
end;

procedure TModelTable.SetTableName(const AValue: String);
begin
  If (AValue=Name) then
    FTableName:=''
  else
    FTableName:=AValue;
end;

constructor TModelTable.Create(AOwner: TComponent);
begin
  FFields:=TModelFieldDefs.Create('');
  inherited Create(AOwner);
end;

destructor TModelTable.Destroy;
begin
  FreeAndnil(FFields);
  inherited Destroy;
end;

{ TModelComponent }

function TModelComponent.GetD(AIndex: integer): integer;
begin
  Result:=FDims[AIndex];
end;

procedure TModelComponent.SetD(AIndex: integer; const AValue: integer);
begin
  FDims[AIndex]:=AValue;
end;

procedure TModelComponent.InvalidateRect(Erase : Boolean);

Var
  P : TDDModelPage;

begin
  If Self is TDDModelPage then
    P:=TDDModelPage(Self)
  else if Owner is TDDModelPage then
    P:=TDDModelPage(Owner)
  else
    P:=Nil;
  If Assigned(P) then
    P.DoInvalidateRect(Rect(Left,Top,Left+Width,Top+Height),Erase);
end;

procedure TModelComponent.SetBounds(const ALeft, ATop, AWidth, AHeight: Integer
  );
begin
  InvalidateRect(False);
  FDims[0]:=ATop;
  FDims[1]:=ALeft;
  FDims[2]:=AWidth;
  FDims[3]:=AHeight;
  InvalidateRect(False);
end;

function TModelComponent.HasParent: Boolean;
begin
  Result:=Owner is TModelComponent;
end;

function TModelComponent.GetParentComponent: TComponent;
begin
  if Owner is TModelComponent then
    Result:=Owner
  else
    Result:=nil;
end;

{ TDDModelPage }

procedure TDDModelPage.GetChildren(Proc: TGetChildProc; Root: TComponent);

var
  i: Integer;

begin
  for i:=0 to ComponentCount-1 do
    Proc(Components[i]);
end;

procedure TDDModelPage.DoInvalidateRect(const ARect: TRect; Erase: boolean);
begin
  if CSLoading in ComponentState then
    exit;
  if Assigned(FOnInvalidateRect) then
    FOnInvalidateRect(Self,ARect,Erase);
end;

{ TModelFieldDef }

{$ifndef oldfields}
function TModelFieldDef.GetDisplayName: string;
begin
  Result:=FieldName;
  If (Result='') then
    Result:=inherited GetDisplayName;
end;
{$endif}
{ TModelFieldDefs }

function TModelFieldDefs.GetF(Aindex : Integer): TModelFieldDef;
begin
  Result:=TModelFieldDef(Items[AIndex]);
end;

procedure TModelFieldDefs.SetF(Aindex : Integer; const AValue: TModelFieldDef);
begin
  Items[AIndex]:=AValue;
end;

{ TModelText }

function TModelText.GetCaption: String;
begin
  Result:=FCaption;
  If Result='' then
    Result:=Name;
end;

procedure TModelText.SetText(const AValue: TStrings);
begin
  if FText=AValue then exit;
  FText.Assign(AValue);
  InvalidateRect(False);
end;

constructor TModelText.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FText:=TStringList.Create;
end;

destructor TModelText.Destroy;
begin
  FreeAndNil(FText);
  inherited Destroy;
end;

end.

